@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """Provides various field types."""
 import re
 import datetime
@@ -8,6 +9,7 @@ from tavi.base.fields import BaseField
 from tavi.documents import EmbeddedDocument
 from tavi.errors import TaviTypeError
 
+
 class BooleanField(BaseField):
     """Represents a boolean field for a Mongo Document. Supports all the
     validations of *BaseField*.
@@ -16,9 +18,10 @@ class BooleanField(BaseField):
     def validate(self, instance, value):
         super(BooleanField, self).validate(instance, value)
 
-        if not isinstance(value, bool):
+        if value and not isinstance(value, bool):
             instance.errors.add(self.name, "must be a valid boolean")
 
+
 class DateTimeField(BaseField):
     """Represents a naive datetime for a Mongo Document.
     Supports all the validations in *BaseField*.
@@ -31,6 +34,7 @@ class DateTimeField(BaseField):
         if None != value and not isinstance(value, datetime.datetime):
             instance.errors.add(self.name, "must be a valid date and time")
 
+
 class FloatField(BaseField):
     """Represents a floating point number for a Mongo Document.
 
@@ -50,19 +54,25 @@ class FloatField(BaseField):
         """Validates the field."""
         super(FloatField, self).validate(instance, value)
 
-        if isinstance(value, int):
-            value = float(value)
+        if None != value:
+            if isinstance(value, int):
+                value = float(value)
 
-        if not isinstance(value, float):
-            instance.errors.add(self.name, "must be a float")
+            if not isinstance(value, float):
+                instance.errors.add(self.name, "must be a float")
 
-        if not None == self.min_value and value < self.min_value:
-            instance.errors.add(self.name,
-                "is too small (minimum is %s)" % self.min_value)
+            if not None == self.min_value and value < self.min_value:
+                instance.errors.add(
+                    self.name,
+                    "is too small (minimum is %s)" % self.min_value
+                )
+
+            if not None == self.max_value and value > self.max_value:
+                instance.errors.add(
+                    self.name,
+                    "is too big (maximum is %s)" % self.max_value
+                )
 
-        if not None == self.max_value and value > self.max_value:
-            instance.errors.add(self.name,
-                "is too big (maximum is %s)" % self.max_value)
 
 class IntegerField(BaseField):
     """Represents a integer number for a Mongo Document.
@@ -83,16 +93,22 @@ class IntegerField(BaseField):
         """Validates the field."""
         super(IntegerField, self).validate(instance, value)
 
-        if not isinstance(value, int):
-            instance.errors.add(self.name, "must be a integer")
+        if None != value:
+            if not isinstance(value, int):
+                instance.errors.add(self.name, "must be a integer")
 
-        if not None == self.min_value and value < self.min_value:
-            instance.errors.add(self.name,
-                "is too small (minimum is %s)" % self.min_value)
+            if not None == self.min_value and value < self.min_value:
+                instance.errors.add(
+                    self.name,
+                    "is too small (minimum is %s)" % self.min_value
+                )
+
+            if not None == self.max_value and value > self.max_value:
+                instance.errors.add(
+                    self.name,
+                    "is too big (maximum is %s)" % self.max_value
+                )
 
-        if not None == self.max_value and value > self.max_value:
-            instance.errors.add(self.name,
-                "is too big (maximum is %s)" % self.max_value)
 
 class ObjectIdField(BaseField):
     """Represents an Object Id generated by Mongo. Supports all the validations
@@ -118,6 +134,7 @@ class ObjectIdField(BaseField):
             if not isinstance(value, ObjectId):
                 instance.errors.add(self.name, "must be a valid Object Id")
 
+
 class StringField(BaseField):
     """Represents a String field for a Mongo Document.
 
@@ -136,17 +153,23 @@ class StringField(BaseField):
                   pattern; default is *None*
 
     """
-    def __init__(self, name, length=None, min_length=None, max_length=None,
-            pattern=None, **kwargs):
+    def __init__(
+        self, name,
+        length=None, min_length=None, max_length=None, pattern=None,
+        **kwargs
+    ):
+
         super(StringField, self).__init__(name, **kwargs)
 
-        self.length     = length
+        self.length = length
         self.min_length = min_length
         self.max_length = max_length
-        self.regex      = re.compile(pattern) if pattern else None
+        self.regex = re.compile(pattern) if pattern else None
 
     def __set__(self, instance, unstripped_value):
         value = unstripped_value.strip() if unstripped_value else None
+        if isinstance(value, str):
+            value = unicode(value, "utf-8")
         super(StringField, self).__set__(instance, value)
 
     def validate(self, instance, value):
@@ -158,20 +181,27 @@ class StringField(BaseField):
             instance.errors.add(self.name, "is required")
 
         if self.length and self.length != val_length:
-            instance.errors.add(self.name,
-                "is the wrong length (should be %s characters)" % self.length)
+            instance.errors.add(
+                self.name,
+                "is the wrong length (should be %s characters)" % self.length
+            )
 
         if self.min_length and val_length < self.min_length:
-            instance.errors.add(self.name,
-                "is too short (minimum is %s characters)" % self.min_length)
+            instance.errors.add(
+                self.name,
+                "is too short (minimum is %s characters)" % self.min_length
+            )
 
         if self.max_length and val_length > self.max_length:
-            instance.errors.add(self.name,
-                "is too long (maximum is %s characters)" % self.max_length)
+            instance.errors.add(
+                self.name,
+                "is too long (maximum is %s characters)" % self.max_length
+            )
 
-        if self.regex and self.regex.match(value) is None:
+        if self.regex and value and self.regex.match(value) is None:
             instance.errors.add(self.name, "is in the wrong format")
 
+
 class EmbeddedField(BaseField):
     """Represents an embedded Mongo document. Raises a TaviTypeError if *doc*
     is not a tavi.document.EmbeddedDocument.
@@ -183,10 +213,10 @@ class EmbeddedField(BaseField):
 
         if not isinstance(doc_instance, EmbeddedDocument):
             raise TaviTypeError(
-                    "expected %s to be a subclass of "
-                    "tavi.document.EmbeddedDocument" %
-                    doc_instance.__class__
-                    )
+                "expected %s to be a subclass of "
+                "tavi.document.EmbeddedDocument" %
+                doc_instance.__class__.__name__
+            )
 
         self.doc_class = doc
         self.value = self.default or doc_instance
@@ -196,7 +226,16 @@ class EmbeddedField(BaseField):
 
     def __set__(self, instance, value):
         if value:
-            if not self.value: self.value = self.doc_class()
+            if not isinstance(value, EmbeddedDocument):
+                raise TaviTypeError(
+                    "expected %s to be a subclass of "
+                    "tavi.document.EmbeddedDocument" %
+                    value.__class__
+                )
+
+            if not self.value:
+                self.value = self.doc_class()
+
             for field in value.fields:
                 embedded_value = getattr(value, field, None)
                 setattr(self.value, field, embedded_value)
@@ -206,9 +245,17 @@ class EmbeddedField(BaseField):
 
 class ListField(BaseField):
     """Represents a list of embedded document fields."""
+    def __init__(self, name, type_, **kwargs):
+        super(ListField, self).__init__(name, **kwargs)
+        self._type = type_
+
     def __get__(self, instance, owner):
         if self.attribute_name not in instance.__dict__:
-            setattr(instance, self.attribute_name, EmbeddedList(self.name))
+            setattr(
+                instance,
+                self.attribute_name,
+                EmbeddedList(self.name, self._type)
+            )
         return getattr(instance, self.attribute_name)
 
     def __set__(self, instance, value):
